/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.results;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import monalisa.Project;
import monalisa.data.pn.PetriNet;
import monalisa.data.pn.TInvariant;
import monalisa.data.pn.Transition;

public final class Mcts implements Result, Collection<TInvariant> {
    private static final long serialVersionUID = -2642935086875477004L;
    private final List<TInvariant> mcts;

    public Mcts(List<TInvariant> mcts) {
        this.mcts = mcts;
    }

    @Override
    public void export(File path, Configuration config, Project project) throws IOException {
        MctsConfiguration mctsConfig = (MctsConfiguration) config;
        String comment = mctsConfig.isStrong() ? "# Enzyme Subsets (occurrence-oriented)" : "# Maximal Common Transition Sets (support-oriented)";
        mctSetExport(project, comment, path, mcts);
    }

    @Override
    public String filenameExtension() {
        return "mct";
    }
    
    @Override
    public String toString() {
        return mcts.toString();
    }

    @Override
    public boolean add(TInvariant o) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean addAll(Collection<? extends TInvariant> c) {
        throw new UnsupportedOperationException();
    }

    @Override
    public void clear() {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean contains(Object o) {
        return mcts.contains(o);
    }

    @Override
    public boolean containsAll(Collection<?> c) {
        return mcts.containsAll(c);
    }

    @Override
    public boolean isEmpty() {
        return mcts.isEmpty();
    }

    @Override
    public Iterator<TInvariant> iterator() {
        return mcts.iterator();
    }

    @Override
    public boolean remove(Object o) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean removeAll(Collection<?> c) {
        throw new UnsupportedOperationException();
    }

    @Override
    public boolean retainAll(Collection<?> c) {
        throw new UnsupportedOperationException();
    }

    @Override
    public int size() {
        return mcts.size();
    }

    @Override
    public Object[] toArray() {
        return mcts.toArray();
    }

    @Override
    public <T> T[] toArray(T[] a) {
        return mcts.toArray(a);
    }
    
    private static List<Transition> sortedTransitions(PetriNet petriNet) {
        ArrayList<Transition> result = new ArrayList<Transition>(petriNet.transitions());
        Collections.sort(result);
        return result;
    }

    private static void mctSetExport(Project project, String comment, File file, List<TInvariant> mctSet) {
        PrintWriter out = null;
        try {
            out = new PrintWriter(file);
            List<Transition> transitions = sortedTransitions(project.getPetriNet());
            out.println(comment);
            Transition transition;
            out.println("\n#List of all sets with their containing transitions (by internal ID)");            
            for (TInvariant inv : mctSet) {
                List<Integer> values = inv.asVector(transitions);
                
                out.printf("Set %d(%d): ", inv.id()+1, inv.size());
                               
                for (int i = 0; i < values.size(); i++) {
                    if (values.get(i) == 0)
                        continue;
                    transition = transitions.get(i);
                    out.printf("%d ", transition.id()+1);
                }
                out.print("\n");
            }
            
            out.println("\n\n#List of all sets with their containing transitions (by name)");
            
            for (TInvariant inv : mctSet) {
                List<Integer> values = inv.asVector(transitions);
                
                out.printf("Set %d(%d): ", inv.id()+1, inv.size());
                               
                for (int i = 0; i < values.size(); i++) {
                    if (values.get(i) == 0)
                        continue;
                    transition = transitions.get(i);
                    out.printf("%s ", ((String) transition.getProperty("name")).replace(" ", "_"));
                }
                out.print("\n");
            }            
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        finally {
            if (out != null)
                out.close();
        }
    }
}