/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import monalisa.tools.cluster.ClusterTool;
import monalisa.tools.distance.DistanceTool;
import monalisa.tools.mcs.McsTool;
import monalisa.tools.mcts.MctsTool;
import monalisa.tools.tinv.TInvariantTool;

/**
 * The different types of Petri net tools that may be run.
 * @author Konrad Rudolph
 */
public enum ToolType {
    TInvariants(TInvariantTool.class),
    MaximalCommonTransitionSets(MctsTool.class, TInvariants),
    TInvariantDistanceMatrix(DistanceTool.class, TInvariants),
    MinimalCutSets(McsTool.class, TInvariants),
    TInvariantCluster(ClusterTool.class, TInvariantDistanceMatrix);
    
    private final Class<? extends Tool> toolClass;
    private final List<ToolType> dependencies;
    
    ToolType(Class<? extends Tool> toolClass, ToolType... dependencies) {
        this.toolClass = toolClass;
        this.dependencies = Arrays.asList(dependencies);
    }
    
    public List<ToolType> dependencies() {
        return dependencies;
    }
    
    public boolean dependsOn(ToolType other) {
        for (ToolType tt : dependencies)
            if (tt == other)
                return true;
        return false;
    }
    
    public List<ToolType> getDependentTools(ToolType tool) {
        List<ToolType> ret = new ArrayList<ToolType>();
        for (ToolType dependent : values())
            if (dependent.dependsOn(tool))
                ret.add(dependent);
        
        return ret;
    }
    
    Tool createTool() throws InstantiationException, IllegalAccessException {
        return (Tool) toolClass.newInstance();
    }
}
