/*
 *
 *  This file ist part of the software MonaLisa.
 *  MonaLisa is free software, dependend on non-free software. For more information read LICENCE and README.
 *
 *  (c) Molekulare Bioinformatik, Goethe University Frankfurt, Frankfurt am Main, Germany
 *
 */

package monalisa.tools.knockout;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import monalisa.data.Pair;
import monalisa.data.pn.PetriNet;
import monalisa.data.pn.Transition;
import monalisa.data.pn.UniquePetriNetEntity;

public class DoubleTransitionKnockout extends KnockoutAlgorithm {
    private List<Pair<Transition, Transition>> pairs
        = new ArrayList<Pair<Transition, Transition>>();
    private Iterator<Pair<Transition, Transition>> pairIterator;
    private List<UniquePetriNetEntity> currentKnockouts;
    
    DoubleTransitionKnockout(PetriNet pn) {
        super(pn);
        List<Transition> transitions = new ArrayList<Transition>();
        for (Transition t : pn.transitions())
            transitions.add(t);
        for (int i = 0; i < transitions.size(); i++)
            for (int j = i + 1; j < transitions.size(); j++)
                pairs.add(Pair.of(transitions.get(i), transitions.get(j)));
        
        pairIterator = pairs.iterator();
    }

    @Override
    protected PetriNet getNextKnockOutNetwork() {
        Pair<Transition, Transition> p = pairIterator.next();
        PetriNet copy = new PetriNet(getPetriNet());
        
        currentKnockouts = new ArrayList<UniquePetriNetEntity>();
        currentKnockouts.add(p.first());
        currentKnockouts.add(p.second());
        
        copy.removeTransition(p.first());
        copy.removeTransition(p.second());
        return copy;
    }

    @Override
    protected boolean hasNextKnockOutNetwork() {
        return pairIterator.hasNext();
    }
    
    protected List<UniquePetriNetEntity> getKnockoutEntities() {
        return currentKnockouts;
    }

    @Override
    protected int getTotalKnockouts() {
        return pairs.size();
    }
}
